# Author: Kero van Gelder
# Copyright: Kero van Gelder, the GNU Lesser General Public License applies
# First version: 17 Jul 2001

require 'kero/xml_re'
require 'agenda/appointment'
require 'agenda/repetition'

Home = ENV["HOME"]
Config = Home + "/.rbagenda.cfg"
OldDataFile = Home + "/.rbagenda.xml"
HistDataFile = Home + "/.rb-pim/agenda-hist.xml"
FutureDataFile = Home + "/.rb-pim/agenda-future.xml"

module Agenda
  class Database
    attr_reader :appointments  # set of Appointment's
    def initialize()
      @appointments = []
      @changed = false
    end

    def changed!
      @changed = true
    end

    def changed?
      @changed
    end

    def add(appt)
      addChild(appt)
      @changed = true
    end

    def delete(appt)
      if appt.instance_of? Deviation
	#appt.repeating.delete_deviation(appt)
      else
	@appointments.delete(appt)
	@changed = true
      end
    end

    def match(interval)
      @appointments.collect { |appt|
	appt.match(interval)
      }.flatten()
    end

    include XmlElt
    def Database::xmlName() "agenda" end
    def addChild(appt)
      @appointments << appt
    end

    def dump()
      "<agenda>\n"+
      appointments.collect { |appt|
	appt.dump()
      }.join("\n")+"\n"+
      "</agenda>\n"
    end

    def save(filename)
      if changed?  # lock if >1 backend...
	File.rename(filename, filename+".old")  if File.exists?(filename)
	File.open(filename, "w") { |f|
	  f.print(dump)
	}
	@changed = false
      end
    end      
  end

  class Agenda
    def initialize(split_date, hist, future)
      @future = TimeInterval.new(split_date, 1<<30)
      @history_db = hist
      @history_db = Database.new()  unless @history_db
      @future_db = future
      @future_db = Database.new()  unless @future_db
#      @future_db.appointments.dup.each { |appt|
#	if appt.match(@future).length == 0
#	  @future_db.delete(appt)
#	  @history_db.add(appt)
#	end
#      }
    end

    def match(interval)
      result = []
      if @future.overlap? interval
	result.concat(@future_db.match(interval))
	if interval < @future
	  result.concat(@history_db.match(interval))
	end
      else
	result.concat(@history_db.match(interval))
      end
      result
    end

    def add(appt)
      db = (appt.match(@future).length > 0) ? @future_db : @history_db
      db.add(appt)
    end

    def delete(appt)
      db = (appt.match(@future).length > 0) ? @future_db : @history_db
      db.delete(appt)
    end

    def changed(appt)
      db = (appt.match(@future).length > 0) ? @future_db : @history_db
      db.changed!      
    end

    def save()
      @history_db.save(HistDataFile)
      @future_db.save(FutureDataFile)
    end
  end


  @parser = XmlParser.new([Database, Appointment, Repetition, Deviation])

p Time.now
  if File.exists?(FutureDataFile)
    File.open(FutureDataFile, "r") { |f|
      @future = @parser.parseStream(f)[0]
    }
    if File.exists?(HistDataFile)
      File.open(HistDataFile, "r") { |f|
	@history = @parser.parseStream(f)[0]
      }
    end
  elsif File.exists?(OldDataFile)
    File.open(OldDataFile, "r") { |f|
      @future = @parser.parseStream(f)[0]
    }
  end

  now = Time.now
  today = Time.local(0,0,0, now.day,now.month,now.year, *[nil]*4)
  $agenda = Agenda.new(today, @history, @future)
  $agenda = Agenda::DataFile.new()  unless $agenda
p Time.now
end
