require 'agenda/appointment'

module Agenda
  OpenRepetition = -1

=begin
== Repetetive Appointment
The great advantage of an electronical agenda.
=end
  class Repetition
    def initialize(appt=nil, pattern="")
      @appointment = appt
      @pattern = pattern
      @deviation = {}
      @durati = OpenRepetition # need an associated starting point...
    end
    attr_accessor :appointment, :pattern, :deviation, :durati

    def match(interval)
      start_time = interval.time
      start_time = appointment.time  if interval.time < appointment.time
      end_time = interval.time + interval.duration - 1
      if @durati != OpenRepetition and end_time > appointment.time + durati - 1
	end_time = appointment.time + durati - 1
      end
      # end_time = appointment.rep_end  if appointment.rep_end < end_time
      result = []

      steps = (pattern.length > 1) ? pattern[1].chr.to_i : 1
      step_method = ""
      case pattern[0].chr
      when "D"
	appt_start = appointment.time.nextDayAfter(start_time)
	while ((appt_start - appointment.time + Hour).to_i / Day) % steps != 0
	  appt_start = appt_start.nextDay()
	end
	step_method = "nextDay"
      when "W"
	appt_start = appointment.time.nextWeekAfter(start_time)
	while ((appt_start - appointment.time + Hour).to_i / Week) % steps != 0
	  appt_start = appt_start.nextWeek()
	end
	step_method = "nextWeek"
      when "Y"
	appt_start = appointment.time.nextYearAfter(start_time)
	while (appt_start.year - appointment.time.year) % steps != 0
	  appt_start = appt_start.nextYear()
	end
	step_method = "nextYear"
      else
	# No longer legal
	raise "Unknown repetition pattern"
      end

      while appt_start <= end_time
	#p "Looking for dev[#{appt_start}]"
	#p " -> Looking for dev[#{search_interval.time.inspect}]"
	if dev = @deviation.fetch(appt_start, nil)
	  result << dev  if dev.overlap?(interval)
	  #p "D #{dev.subject}, #{dev.description}"  if dev.overlap?(interval)
	else
	  result << Occurrence.new(self, appt_start)
	  #p "O #{result.last.subject}, #{result.last.description}"
	end
	steps.times { appt_start = appt_start.method(step_method).call() }
      end

      @deviation.each_pair { |start, dev|
	if not interval.overlap? TimeInterval.new(dev.occurrence, dev.duration)
	  result << dev  if interval.overlap?(dev)
	  #p "D3 #{dev.subject}, #{dev.description}"  if dev.overlap?(interval)
	end
      }
      result
    end

    include XmlElt
    def Repetition::xmlName() "rep" end
    def addAttribute(attrib, value=nil)
      case attrib
      when "dur"
	@durati = value.to_i
      when "pattern"
	@pattern = value
      else
	raise "Unexpected attribute #{attrib} for Repetition"
      end
    end
    def addChild(dev)
      # p "Storing dev[#{dev.occurrence}]"
      @deviation.store(dev.occurrence, dev)
      dev.origin = self
    end
    def dump()
      result = %Q(<rep pattern="#{pattern}" dur="#{@durati}">)
      if @deviation.empty?
	result[-1, 0] = "/"
      else
	@deviation.each { |key, dev|
	  result << dev.dump()
	}
	result << "</rep>"
      end
      result
    end
  end

=begin
== Occurence
Single occurence of a repeating appointment. Used as element in matching lists.
=end
  class Occurrence < Appointment
    attr_reader :origin
    def appointment() origin.appointment end
    alias Appointment_init initialize
    def initialize(repeating, start)
      @origin = repeating
      Appointment_init(start, @origin.appointment.duration)
    end
    def subject()  @origin.appointment.subject()  end
    def location()  @origin.appointment.location()  end
    def description()  @origin.appointment.description()  end
    def start() @time end
  end

=begin
==Deviation
Single occurrence of a repeating appointment that deviates from the default
repetition in some way. At least one field, but possibly all fields can be
changed. The repeating appointment and the starting time of the original
occurrence are the unique (and sufficient) reference required.
=end
  class Deviation < Occurrence
    @@EndOfTime = Time.at(2**31-1)

    attr_accessor :origin
    attr_accessor :occurrence
    # at the time, list the deviating fields, which could be all of them
    # (including repeating?)
    def initialize(repeating=nil, start=nil)
      @occurrence = start  if start
      repeating.addChild(self)  if repeating
      @diff = {}
      @deviates = false  # not @diff.empty?
    end

    attr_accessor :cancel
    def cancel!
      # puts "Cancelling #{subject} on #{occurrence}"
      @cancel = true
    end
    def cancelled?
      @cancel
    end

    def time()
      return @@EndOfTime  if cancelled?
      if @diff.has_key?("time")
	@diff.fetch("time")
      else
	@occurrence
      end
    end

    def time=(new_time)
      if new_time == @occurrence
	@diff.delete("time")
      else
	@diff.store("time", new_time)
      end
    end

    def repeats()  Never; end
    def rep_dur()  -1; end
    def repeats=(x) end  # hum hum...
    def rep_dur=(x) end  # ...will probably move this to a sequence of
			 # associated Repeatings

    def Deviation::build_diff(func, stor)
      eval %Q{
	def #{func}()
	  if @diff.has_key?("#{stor}")
	    @diff.fetch("#{stor}")
	  else
	    appointment.#{func}()
	  end
	end
      }
      eval %Q{
	def #{func}=(new_val)
	  if new_val === appointment.#{func}
	    @diff.delete("#{stor}")
	  else
	    @diff.store("#{stor}", new_val)
	  end
	  @deviates = (not @diff.empty?)
	end
      }
    end

    Deviation::build_diff("duration", "dur")
    alias :regular_dur duration
    def duration()
      return 0  if cancelled?
      regular_dur
    end
    Deviation::build_diff("subject", "sub")
    Deviation::build_diff("location", "loc")
    Deviation::build_diff("description", "descr")

    def Deviation::xmlName() "dev" end
    def addAttribute(attrib, value=nil)
      case attrib
      when "occur"
	@occurrence = Time.at(value.to_i)
      when "time"
	@diff.store("time", Time.at(value.to_i))
      when "dur"
	@diff.store("dur", value.to_i)
      when "sub"
	@diff.store("sub", value)
      when "loc"
	@diff.store("loc", value)
      when "descr"
	@diff.store("descr", value.gsub(/\\n/, "\n"))
      when "cancel"
	cancel!
      end
      @deviates = true  # assumption, but things like @location = value won't
			# work to call the (eval()ed) method :(
    end

    def dump()
      unless @deviates
	return ""
      end
      result = %Q(<dev occur="#{@occurrence.to_i}")
      if cancelled?
	result += " cancel"
      else
	@diff.each_pair { |key, val|
	  if val.instance_of? String
	    result += %Q( #{key}=#{val.inspect})
	  elsif val.instance_of? Time
	    result += %Q( #{key}="#{val.to_i}")
	  elsif val.kind_of? Numeric
	    result += %Q( #{key}="#{val.to_i}")
	  else
	    raise "Unknown type to save, sorry (check file and backup)"
	  end
	}
      end
      result + "/>"
    end
  end
end
