# Composed Tk Widgets and advanced Tk Usage
# Author: Kero van Gelder
# Copyright: GNU Lesser General Public License

require 'tk'
require 'observer'


if defined?(UseFont)
  Font = TkFont.new(UseFont)

  class TkLabel
    def initialize(parent, args={})
      args.store('font', Font)
      super(parent, args)
    end
    def configure(slot, value=None)
      super(slot, value)
      super('font', Font)
    end
  end

  class TkListbox
    def initialize(parent, args={})
      args.store('font', Font)
      super(parent, args)
    end
  end

  class TkText
    def initialize(parent, args={})
      args.store('font', Font)
      super(parent, args)
    end
  end

  class TkMenu
    alias :old_add add 
    def add(type, keys={})
      keys.store('font', Font)  if type == 'command' or type == 'cascade'
      old_add(type, keys)
    end
  end

  class TkOptionMenubutton
    alias TkOptionMenubutton_init initialize
    def initialize(parent, tkvar, firstval, *values)
      TkOptionMenubutton_init(parent, tkvar, firstval, *values)
      [firstval, *values].each_index { |i|
	entryconfigure(i, 'font', Font)
      }
    end
  end
end

class TkPopupButton < TkButton
  # alias TkButton_init initialize
  def initialize(parent, numbers, args={})
    # if args[0] == nil; args[0] = {}; end
    args.store('padx', 2)
    args.store('pady', 2)
    args.store('font', Font)  if defined?(UseFont)
    super(parent, args)
    @var = args.fetch('textvariable', nil)  unless args == nil
    bind("Button-1") { |evt|
      #disable()
      if @top == nil
	@top = TkToplevel.new()
	@top.overrideredirect(true)
	@top.geometry("+#{evt.x_root-40}+#{evt.y_root-20}")
	count = 0
	b = []
	numbers.each { |nr|
	  if (count == 4)
	    count = 0
	    Tk.grid(*b)
	    b = []
	  end
	  label = TkLabel.new(@top, 'text'=>sprintf("%02d", nr))
	  label.bind("Button-1") { |evt|
	    @var.value = nr  unless @var == nil
	    event_generate("<MenuSet>")
	    @top.grab('release')
	    @top.withdraw() # destroy(); @top = nil # @top.unmap
	    #enable()
	  }
	  b << label
	  count += 1
	}
	Tk.grid(*b)  unless b.length == 0
      else
	@top.geometry("+#{evt.x_root-40}+#{evt.y_root-20}") #  state('normal')
	@top.deiconify
	@top.raise
      end
      @top.bind("Button-1") { |evt|
	if @top
	  @top.grab('release')
	  @top.withdraw # destroy(); @top = nil # unmap()
	end
	event_generate("ButtonRelease-1")
      }
      @top.bind("Expose") { @top.grab('global') }
    }
  end

  def configure(args)
    v = args.fetch('textvariable', nil)
    @var = v  unless v == nil
    super(args)
  end

  def textvariable(v)
    @var = v
    super(v)
  end
end

require 'agenda/time'
class TkTime < TkFrame
  include Observable

  @@dowName = [ 'Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat' ]
  @@monthName = [
    'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
    'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec',
  ]

  # alias TkFrame_init initialize
  def initialize(parent, *args)
    accuracy = args[0].delete('show')  if args[0]
    super(parent, *args)

    @dow = TkVariable.new()
    dow = TkOptionMenubutton.new(self, @dow, *@@dowName)
    dow.pack('side'=>'left')
    dow.configure('indicator'=>false, 'padx'=>2, 'pady'=>2, 'textvariable'=>@dow)
    dow.menu.bind("<MenuSelect>") {
      newDay = @@dowName.index(@dow.value)
      if newDay != @time.wday
	changed
	@time += ((newDay - @time.wday + 7) % 7) * Day
	displayTime()
	notify_observers(self)
      end
    }

    @day = TkVariable.new()
    day = TkPopupButton.new(self, (1..31), 'textvariable'=>@day)
    day.pack('side'=>'left')
    day.bind("<MenuSet>") { newTime() }

    @month = TkObservable.new()
    month = TkOptionMenubutton.new(self, @month, *@@monthName)
    month.pack('side'=>'left')
    month.configure('indicator'=>false, 'padx'=>2, 'pady'=>2)
    month.menu.bind("<MenuSelect>") { newTime() }
    #@month.add_observer(parent)

    @year = TkVariable.new()
    year = TkEntry.new(self, 'textvariable'=>@year, 'width'=>'4')
    year.pack('side'=>'left')
    year.bind("Any-Key-Return") { newTime() }
    year.bind("FocusOut") { newTime() }

    @hour = TkVariable.new()
    hour = TkPopupButton.new(self, (0...24), 'textvariable'=>@hour)
    hour.pack('side'=>'left')  unless accuracy === "days"
    hour.bind("<MenuSet>") { newTime() }

    @min = TkVariable.new()
    min = TkOptionMenubutton.new(self, @min, *((0..59).select{
						 |i|i%5==0
					       }.collect{
						 |i|sprintf("%02i",i)
					       }))
    min.pack('side'=>'left')  unless accuracy === "days"
    min.configure('indicator'=>false, 'padx'=>2, 'pady'=>2)
    min.menu.bind("<MenuSelect>") { newTime() }
  end

  def newTime()
    new_time = Time.local(0, @min.to_i, @hour.to_i,
			  @day.to_i, @month.value, @year.to_i,
			  *[nil]*4)
    if new_time != @time
      @time = new_time
      @dow.value = @@dowName[@time.wday]
      displayTime()
      changed
      notify_observers(self)
    end
  end

  def time()
    #mon = @@monthName.index(@month.value)
    @time
  end

  def time=(time)
    @time = time
    displayTime()
  end

  def displayTime()
    @dow.value = @@dowName[@time.wday]
    @day.value = @time.day
    @month.value = @@monthName[@time.month-1]
    @year.value = @time.year
    @hour.value = sprintf("%02i", @time.hour)
    @min.value = sprintf("%02i", @time.min)
  end
end


# TkObservable is a nice idea, but doesn't work so far :(
class TkObservable < TkVariable
  include Observable
  def value=(val)
    #p "assigned #{val}"
    changed  if "#{val}" <=> value()
    super
    notify_observers(Time.now(), val)
  end
end
