# Author: Kero van Gelder
# Copyright: Kero van Gelder, the GNU Lesser General Public License applies
# First version: 17 Jul 2001

require 'agenda/database'

begin
  load Config
rescue LoadError
  puts "#{Config} not found, using defaults"
end

require 'agenda/tkedit'
require 'agenda/tkday'
require 'agenda/tkweek'

module Agenda
  class TkAgenda < TkFrame
    attr_reader :agenda, :current
    alias TkFrame_init initialize
    def initialize(parent, agenda, *args)
      TkFrame_init(parent, *args)
      @agenda = agenda
      setView()
    end

    def setView(t=nil)
      @view = TkViewer.new(self)  unless @view
      if @frame != @view
	@frame.pack_forget()  if @frame != nil
	@frame = @view
	@frame.pack('fill'=>'both', 'expand'=>true)
	@frame.show()
      end
    end

    def setEdit(appt)
      @new_appt = (appt == nil)
      @edit_appt = (@new_appt) ? Appointment.new(@frame.time) : appt
      @edit = TkEdit.new(self, method("doneEditing"))  unless @edit
      @edit.showAppointment(@edit_appt)
      if @frame != @edit
	@frame.pack_forget()  if @frame != nil
	@frame = @edit
	@frame.pack('fill'=>'both', 'expand'=>true)
      end
    end

    def doneEditing(edited)
      if edited
	@edit.fillAppointment(@edit_appt)
	agenda.add(@edit_appt)  if @new_appt
        agenda.changed(@edit_appt)
      end
      setView()
    end

    def deleteAppointment(appt)
      agenda.delete(appt)
      @frame.show()
    end

    def saveDatabase()
      $agenda.save()
    end
  end
end

root = TkRoot.new() { title "Ruby Agenda (v0.9.2, by Kero, GLGPL applies)" }
tkagenda = Agenda::TkAgenda.new(root, $agenda)
tkagenda.pack('fill'=>'both', 'expand'=>true)

root.bind("Control-Key-q") { root.destroy() }

Tk.mainloop()

tkagenda.saveDatabase()
