# Author: Kero van Gelder
# Copyright: Kero van Gelder, the GNU Lesser General Public License applies
# First version: 17 Jul 2001

require "agenda/appointment"
require "agenda/tk"

module Agenda
=begin
== Edit an Appointment
Tk interface (frame) to edit an appointment. Since you can also edit fresh,
uninitialized appointments, this serves as the frame to add an appointment as
well.
=end
  class TkEdit < TkFrame
    @@repValue = [ "", "D", "W", "Y" ]  # "M" ?
    @@repName = [ "Never", "Day", "Week", "Year" ]  # "Month" ?


    def update(field)
      if field == @fromEntry
	@untilEntry.time = @fromEntry.time + @duration
      elsif field == @untilEntry
	if @untilEntry.time <= @fromEntry.time
	  @untilEntry.time = @fromEntry.time + 1
	end
	@duration = @untilEntry.time - @fromEntry.time
      else
	# @rep_until
      end
    end

    alias TkFrame_init initialize
    def initialize(parent, callback, *args)
      TkFrame_init(parent, *args)
      @doneEditing = callback
      frame = TkFrame.new(self).pack('anchor'=>'w')
      TkLabel.new(frame, 'text'=>"From").pack('side'=>'left')
      @fromEntry = TkTime.new(frame).pack('side'=>'left')
      @fromEntry.add_observer(self)
      frame = TkFrame.new(self).pack('anchor'=>'w')
      @term = TkVariable.new()
      term = TkOptionMenubutton.new(frame, @term, *["Until", "All Day"])
      term.configure('indicator'=>false, 'padx'=>2, 'pady'=>2)
      @old_term = @term.value = "Until"
      term.pack('side'=>'left')
      term.menu.bind("<MenuSelect>") { |evt|
	if @old_term != @term.value
	  if @term.value == "Until"
	    @untilEntry.pack()
	  else
	    @untilEntry.pack_forget()
	  end
	  @old_term = @term.value
	end
      }
      @untilEntry = TkTime.new(frame).pack('side'=>'left')
      @untilEntry.add_observer(self)

      frame = TkFrame.new(self).pack('anchor'=>'w')
      label = TkLabel.new(frame, 'text'=>"Every").pack('side'=>'left')
      @every = TkVariable.new()
      rep = TkOptionMenubutton.new(frame, @every, *(1..6).collect{|i|i.to_s})
      rep.pack('side'=>'left')
      rep.configure('indicator'=>false, 'padx'=>2, 'pady'=>2)
      @repetition = TkVariable.new()
      rep = TkOptionMenubutton.new(frame, @repetition, *@@repName)
      rep.pack('side'=>'left')
      rep.configure('indicator'=>false, 'padx'=>2, 'pady'=>2)
      @rep_until_type = TkVariable.new()
      @until_type = "endless"
      rep = TkOptionMenubutton.new(frame, @rep_until_type, "endless", "until")
      rep.menu.bind("<MenuSelect>") { |evt|
	if @until_type != @rep_until_type.value
	  if @rep_until_type.value == "until"
	    @rep_until.pack('side'=>'left')
	    @rep_until.time = @untilEntry.time()
	  else
	    @rep_until.pack_forget()
	  end
	  @until_type = @rep_until_type.value
	end
      }
      rep.pack('side'=>'left')
      rep.configure('indicator'=>false, 'padx'=>2, 'pady'=>2)
      @rep_until = TkTime.new(frame, 'show'=>'days')
      @rep_until.add_observer(self)

      frame = TkFrame.new(self).pack('fill'=>'x')
      label = TkLabel.new(frame, 'text'=>"Subject").pack('side'=>'left')
      @subject = TkVariable.new()
      subject = TkEntry.new(frame, 'textvariable'=>@subject)
      subject.pack('side'=>'left', 'fill'=>'x', 'expand'=>true)

      frame = TkFrame.new(self).pack('fill'=>'x')
      label = TkLabel.new(frame, 'text'=>"Location").pack('side'=>'left')
      @location = TkVariable.new()
      location = TkEntry.new(frame, 'textvariable'=>@location)
      location.pack('side'=>'left', 'fill'=>'x', 'expand'=>true)

      @description = TkText.new(self, 'width'=>40, 'height'=>5, 'wrap'=>'word')
      @description.pack('fill'=>'both', 'expand'=>true)

      buttons = TkFrame.new(self)
      buttons.pack('side'=>'bottom', 'fill'=>'x', 'before'=>@description)
      @done = TkButton.new(buttons, 'text'=>'Done', 'padx'=>2, 'pady'=>2)
      @done.command {
	@doneEditing.call(true)
      }
      @done.pack('side'=>'left')
      @cancel = TkButton.new(buttons, 'text'=>'Cancel', 'padx'=>2, 'pady'=>2)
      @cancel.command {
	@doneEditing.call(false)
      }
      @cancel.pack('side'=>'left')
    end

    def showAppointment(appt)
      @fromEntry.time = appt.time
      if appt.duration == AllDay
	@duration = 30*Minute
	@untilEntry.pack_forget()
	@term.value = "All Day"
      else
	@duration = appt.duration
	@untilEntry.pack()
	@term.value = "Until"
      end
      @untilEntry.time = appt.time + @duration
      pattern = appt.repetition[0].pattern  if appt.repetition and appt.repetition[0]
      if pattern
	@repetition.value = @@repName[@@repValue.index(pattern[0, 1])]
	@every.value = (pattern[1, 1] != "") ? pattern[1, 1] : 1
	if appt.repetition[0].durati == OpenRepetition
	  @rep_until_type.value = "endless"
	  @rep_until.pack_forget
	else
	  @rep_until_type.value = "until"
	  @rep_until.pack()
	  @rep_until.time = appt.time + appt.repetition[0].durati
	end
      else
	@repetition.value = "Never"
	@every.value = 1
	@rep_until_type.value = "endless"
	@rep_until.pack_forget
      end
      @subject.value = appt.subject
      @location.value = appt.location
      @description.delete("0.0", 'end')
      @description.insert('end', appt.description)
    end

    def fillAppointment(appt)
      appt.time = @fromEntry.time()
      if @term.value == "Until"
	appt.duration = @untilEntry.time()-appt.time
      else
	appt.duration = AllDay
      end
      if @repetition.value == "Never"
	# invent a (un)setRepetition function ?
	# Well, actually, we'll do this different anyway (multiple reps that
	# can be killed, each)
	# Then this "Never" case dies anyway!!!
      else
	appt.repetition[0] = Repetition.new(appt)  unless appt.repetition[0]
	appt.repetition[0].pattern = @repetition.value[0, 1]
	appt.repetition[0].pattern += @every.value.to_s  if @every.value.to_i > 1
	if @rep_until_type.value == "endless"
	  appt.repetition[0].durati = OpenRepetition  ## <<==-- !!
	else
	  end_time = @rep_until.time()
	  end_time = Time.local(end_time.year, end_time.month, end_time.day,
				23, 59, 59)
	  appt.repetition[0].durati = end_time - @fromEntry.time()
	end
      end
      appt.subject = @subject.value
      appt.location = @location.value
      appt.description = @description.get("0.0", 'end-1char')
    end
  end
end
