require 'tk'

module Agenda
  class TkViewer < TkFrame
    def initialize(tkagenda, *args)
      super(tkagenda, *args)
      @tkagenda = tkagenda
      @agenda = @tkagenda.agenda

      @top_bar = TkFrame.new(self).pack('side'=>'top', 'fill'=>'x')
      succ = TkButton.new(@top_bar, 'text'=>'>', 'padx'=>2, 'pady'=>2)
      succ.pack('side'=>'right', 'fill'=>'x')
      succ.command(proc {
	@view.later()
      })
      today = TkButton.new(@top_bar, 'text'=>'Today', 'padx'=>2, 'pady'=>2)
      today.pack('side'=>'right', 'fill'=>'x')
      today.command(proc {
	@view.show(Time.now())
      })
      pred = TkButton.new(@top_bar, 'text'=>'<', 'padx'=>2, 'pady'=>2)
      pred.pack('side'=>'right', 'fill'=>'x')
      pred.command(proc {
	@view.earlier()
      })

      @bot_bar = TkFrame.new(self).pack('side'=>'bottom', 'fill'=>'x')
      new = TkButton.new(@bot_bar, 'text'=>'New Appt', 'padx'=>2, 'pady'=>2)
      new.pack('side'=>'left', 'fill'=>'x', 'expand'=>true)
      new.command(proc { @tkagenda.setEdit(nil) })
      save = TkButton.new(@bot_bar, 'text'=>"Save DB", 'padx'=>2, 'pady'=>2)
      save.pack('side'=>'left', 'fill'=>'x', 'expand'=>true)
      save.command(proc { @tkagenda.saveDatabase() })

      @views = { "Day"=>TkDay.new(self), "Week"=>TkWeek.new(self) }

      @view_switch = TkVariable.new()
      type = TkOptionMenubutton.new(@top_bar, @view_switch, *@views.keys)
      @view_switch.value = "Day"
      type.pack('side'=>'left')
      type.configure('indicator'=>false, 'padx'=>2, 'pady'=>2, 'textvariable'=>@view_switch)
      type.menu.bind("<MenuSelect>") {
	if @view_switch.value != @old_view
	  @time = @view.time
	  show(true)
	end
      }

      @time = Time.now()
      show(true)
    end
    attr_reader :top_bar, :bot_bar, :tkagenda, :agenda

    def show(switch_view=false)
      if switch_view
	@old_view = @view_switch.value
	@view.remove_from()  if @view
	@view = @views.fetch(@old_view)
	@view.display_on(self)
	@view.show(@time)
      else
	@view.show(@view.time)
      end
    end

    def time()
      @view.time
    end
  end

  class TkView
    def initialize(parent, *args)
      @parent = parent
      @frame = TkFrame.new(parent)

      @list = TkListbox.new(@frame, 'height'=>6)
      @list.bind("Any-ButtonPress") { |evt|
	# hack to let Tk update its selection, before we deal with it
	TkAfter.new(1, 1, proc {setSelectedAppt}).start
	setup_popup(evt.x_root, evt.y_root)
      }
      @list.bind("Any-ButtonRelease") { stop_popup() }
      # @list.bind("Any-KeyRelease") { setSelectedAppt } # no keys in use...
      @list.pack('fill'=>'both', 'expand'=>true)

      @location = TkLabel.new(@frame, 'text'=>"Location: ")
      @location.pack('anchor'=>'w')

      @description = TkText.new(@frame, 'width'=>40, 'height'=>4, 'takefocus'=>false)
      @description.pack('fill'=>'both', 'expand'=>true)
    end

    def setTimeFrame(interval)
      @interval = interval
      @list.delete(0, 'end')
      @appt_list = @parent.agenda.match(interval).sort
      @appt_list.each { |appt|
	@list.insert('end', "#{print_appt(interval, appt)}#{appt.subject}")
      }
      @list.selection_set(0, 0)
      setSelectedAppt()
    end
    attr_reader :interval, :time  # for creating new appts

    def setSelectedAppt()
      sel = @list.curselection
      @description.state('normal')
      @description.delete("0.0", 'end')
      if sel != nil and sel[0] != nil
	appt = @appt_list[sel[0]]
	@location.text("Location: #{appt.location}")
	@description.insert('end', appt.description)
      else
	@location.text("Location: ")
      end
      @description.state('disabled')
    end

    def setup_popup(x, y)
      @after = TkAfter.new(250, 1, proc { popup(x, y) })
      @after.start
    end

    def stop_popup()
      @after.stop  if @after  # mutex'd...
    end

    def popup(x, y)
      if not @popup
	@popup = TkMenu.new() { tearoff false }
	@popup.add('command', 'label'=>"Edit appointment", 'command'=>proc { edit_appt })
	@popup.add('command', 'label'=>"Delete appointment", 'command'=>proc { del_appt })
	@popup.add('separator')
	@popup.add('command', 'label'=>"edit Occurrence", 'command'=>proc { edit_occur })
	@popup.add('command', 'label'=>"Cancel occurrence", 'command'=>proc { del_occur })
      end
      @popup.popup(x, y)
    end

    def edit_appt()
      sel = selected
      @parent.tkagenda.setEdit(sel.appointment)  if sel
    end

    def del_appt()
      sel = selected
      @parent.tkagenda.deleteAppointment(sel)  if sel
    end

    def edit_occur()
      occ = selected
      if occ.instance_of? Deviation or occ.instance_of? Appointment
	@parent.tkagenda.setEdit(occ)
      elsif occ
	@parent.tkagenda.setEdit(Deviation.new(occ.origin, occ.start))
      end
    end

    def del_occur()
      sel = selected
      if sel.kind_of? Occurrence
	if not sel.kind_of? Deviation
	  sel = Deviation.new(sel.origin, sel.start)
	  sel.cancel!
	  setTimeFrame(@interval)
	end
      end
    end

    def selected()
      series = @list.curselection
      index = series[0]  if series
      return @appt_list[index]  if index
      return nil
    end

    def print_time(interval, appt)
      return (appt.duration == AllDay) ? "" : appt.time.strftime("%H:%M")
    end

    def print_appt(interval, appt)
      time_str = print_time(interval, appt)
      result = appt.time.strftime("%d %b %y")
      result += ", " + time_str  if time_str != ""
      return result
    end
  end
end
