# Author: Kero van Gelder
# Copyright: Kero van Gelder, the GNU Lesser General Public License applies
# First version: 17 Jul 2001

Home = ENV["HOME"]
Config = Home + "/.rbagenda.cfg"
DataFile = Home + "/.rbagenda.xml"

begin
  load Config
rescue LoadError
  puts "#{Config} not found, using defaults"
end

require 'agenda/database'

require 'wise.rb'

module Agenda
  class UI < Window
    attr_reader :agenda, :current
    alias Frame_init initialize
    def initialize(parent, agenda, *args)
      Frame_init(parent, *args)
      @agenda = agenda
      setView()
    end

    def setView(t=nil)
      @view = Viewer.new(self)  unless @view
      if @frame != @view
	@frame.pack_forget()  if @frame != nil
	@frame = @view
	@frame.pack('fill'=>'both', 'expand'=>true)
	@frame.show()
      end
    end

    def setEdit(appt)
      @new_appt = (appt == nil)
      @edit_appt = (@new_appt) ? Appointment.new(@frame.time) : appt
      @edit = Edit.new(self, method("doneEditing"))  unless @edit
      @edit.showAppointment(@edit_appt)
      if @frame != @edit
	@frame.pack_forget()  if @frame != nil
	@frame = @edit
	@frame.pack('fill'=>'both', 'expand'=>true)
      end
    end

    def doneEditing(edited)
      if edited
	@edit.fillAppointment(@edit_appt)
	agenda.add(@edit_appt)  if @new_appt
      end
      setView()
    end

    def deleteAppointment(appt)
      agenda.delete(appt)
      @frame.show()
    end

    def saveDatabase()
      if agenda.changed?
	#p "Dumping agenda:"
	#print agenda.dump()
	#return
	File.rename(Agenda::DataFile, Agenda::DataFile+".old")
	File.open(Agenda::DataFile, "w") { |f|
	  f.print(agenda.dump)
	}
      end
    end
  end
end

agenda = nil
if File.exists?(Agenda::DataFile)
  File.open(Agenda::DataFile, "r") { |f|
    parser = XmlParser.new([Agenda::Database, Agenda::Appointment, Agenda::Repetition, Agenda::Deviation])
    agenda = parser.parseStream(f)[0]
  }
end
if agenda == nil
  agenda = Agenda::DataFile.new()
end

root = Window.new('title'=>"RbAgenda-Wise (v0.9, by Kero, GLGPL applies)")
tkagenda = Agenda::UI.new(root, agenda)

root.bind("Control-Key-q") { root.destroy() }

Serialize::serialize()  if __FILE__ == $0

tkagenda.saveDatabase()
